!debug = 0

if !debug = 0 then
	logmode +addin
endif

'check that an object exists
%type = @getthistype
if %type="NONE" then
	seterr "No object found, please open an Equation object"
	stop
endif

'check that _this object is an equation
if %type<>"EQUATION" then
	seterr "Procedure can only be run from an Equation object"
	stop
endif

'check that equation was estimated by LS, and by list.
%method = _this.@method
if %method <> "LS" then
	seterr "Procedure can only be run from Equations estimated by Least Squares"
	stop
endif
if _this.@bylist = 0 then
	seterr "Procedure can only be run from Equations estimated by list"
	stop
endif

'get dependent variable and regressors
%vars = _this.@varlist
%dep = @word(%vars,1)
%regs = @wmid(%vars,2)

'check no ARMA terms
if _this.@ncoefs <> @wcount(%regs) then
	seterr "Procedure cannot be run on Equations with ARMA terms"
	stop
endif

'get sample string
%origsmpl = @pagesmpl
%smplstr = _this.@smpl
're-estimate equation so we can set the sample to be observations included in the equation
smpl {%smplstr}
%spec = _this.@command
_this.{%spec}
smpl if resid<>na
!estobs = _this.@regobs


'get options for minimum subsample size
!h=0.15
%h = @equaloption("size")
if @len(%h)=0 then
	%h = @str(!h)
	!result = @uidialog("edit",%h, "Enter a minimum sub-sample size (either a number of observations or as a fraction)", "caption", "Bai-Perron test")
	if !result = -1 then
		smpl {%origsmpl}
		stop
	endif
endif

!temp = @val(%h)
'check %h is either a positive integer, or between 0 and 1
if (@floor(!temp) = !temp and !temp>0) or (!temp>0 and !temp<1) then
	!h = !temp
else
	seterr "Invalid sub-sample size given"
	smpl {%origsmpl}
	stop
endif


'open up connection to R
!maxerr = @maxerrcount
setmaxerrs !maxerr+1
if !debug = 1 then
	xopen(r, case=lower)   'open an R session
else
	xopen(r, case=lower, nolog)
endif

%str = @lasterrstr
if (@instr(%str,"StatConnectorSrv")>0) then 
	seterr "R is not installed, please check your EViews documentation for instructions on installing R."
	smpl {%origsmpl}
	stop
endif
setmaxerrs !maxerr

'put dependent variable
xput {%dep}

'build up formula string for R, and send variables over to R
%frml = %dep + " ~ "
for !i=1 to @wcount(%regs)
	%reg = @word(%regs,!i)
	if %reg = "C" then
		%reg = "1"
	else
		xput() {%reg}
		%reg = @xputnames
	endif
	%frml = %frml + " " + %reg + " +"
next
%frml = @left(%frml,@len(%frml)-2)  'trim off the final errant " +"

' install and load the StrucChange package in R
xrun "if (sum(.packages(TRUE)==""strucchange"")==0) install.packages(""strucchange"")"
xrun "library(strucchange)"

'perform breakpoint test
if !estobs>500 then
	statusline "R is performing the breakpoint tests.  This may take some time.  Please wait"
endif
!maxerr = @maxerrcount
setmaxerrs !maxerr+1
%cmd = "bp <- breakpoints(" + @lower(%frml) + ", h=" + @str(!h) + ")"
xrun %cmd
%str = @lasterrstr
if (@instr(%str,"evaluation stopped")>0) then 
	seterr "R returned an error.  The most likely cause of this is that your equation did not have enough observations to perform the test, or that one of the tested breakpoints caused a singularity."
	smpl {%origsmpl}
	stop
endif
setmaxerrs !maxerr

'build BIC cmd
%cmd = "bpaic <- AIC(bp, k=log(bp$nobs))"
xrun %cmd

'retrieve BICs
%bicname = @getnextname("BIC")
xget(name=%bicname) bpaic

'retrieve loglikes
%logname = @getnextname("LOGLIK")
vector {%logname} = {%bicname} 'size it correctly
for !i=1 to @rows({%bicname})
	%temp = @getnextname("_temp")
	%cmd = "logLik(bp," + @str(!i-1) + ")"
	xget(name = %temp) {%cmd}
	{%logname}(!i) = {%temp}
	d {%temp}
next

'calculate RSS
%rssname = @getnextname("RSS")
vector {%rssname} = {%logname}*(-2/!estobs) - 1 - log(2*4*@atan(1)) + log(!estobs)
{%rssname} = exp({%rssname})

'calculate ncoefs
%ncoefs = @getnextname("NCOEFS")
%breakfill = @getnextname("breakfill")
vector {%breakfill} = {%bicname} 'size it correctly
for !i=1 to @rows({%breakfill})
	{%breakfill}(!i) = !i-1
next
vector {%ncoefs} =(_this.@ncoef + 1)*({%breakfill}+1)
d {%breakfill}


!chosenbreaks = @imin({%bicname})-1

if !chosenbreaks>0 then
	'get breakpoints
	%breakname = @getnextname("BREAKS")
	xget(name=%breakname, type=vector) "bp$breakpoints"
endif

%tn = @getnextname("TABLE")
!cols = @rows({%bicname}) +1
table(15,!cols) {%tn}
{%tn}.setwidth(1) 12
{%tn}.setwidth(2:5) 9
!rowcounter=1
{%tn}(!rowcounter,1) = "Bai-Perron breakpoint test"
!rowcounter = !rowcounter + 1
%datestring = "Date: " + @datestr(@now,"MM/DD/YY") + "   Time: " + @datestr(@now,"HH:MM")
{%tn}(!rowcounter,1) = %datestring
!rowcounter=!rowcounter+1
%samplestr = "Sample: " + %smplstr
{%tn}(!rowcounter,1) = %samplestr
!rowcounter=!rowcounter+1
%obsstr = "Included observations: " + @str(!estobs)
{%tn}(!rowcounter,1) = %obsstr
!rowcounter=!rowcounter+1
{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

{%tn}(!rowcounter,1) = "Breakpoints"
{%tn}.setjust(!rowcounter,1) left
for !i=1 to @rows({%bicname})
	{%tn}(!rowcounter,1+!i) = @str(!i-1,"i")
next
!rowcounter = !rowcounter+1
{%tn}(!rowcounter,1) = "BIC"
{%tn}.setjust(!rowcounter,1) left
for !i=1 to @rows({%bicname})
	{%tn}(!rowcounter,1+!i) = {%bicname}(!i)
next
!temp = !chosenbreaks+2
{%tn}.settextcolor(!rowcounter,!temp) blue
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "Log-Lik"
{%tn}.setjust(!rowcounter,1) left
for !i=1 to @rows({%logname})
	{%tn}(!rowcounter,1+!i) = {%logname}(!i)
next
!temp = !chosenbreaks+2
{%tn}.settextcolor(!rowcounter,!temp) blue
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "RSS"
{%tn}.setjust(!rowcounter,1) left
for !i=1 to @rows({%rssname})
	{%tn}(!rowcounter,1+!i) = {%rssname}(!i)
next
!temp = !chosenbreaks+2
{%tn}.settextcolor(!rowcounter,!temp) blue
!rowcounter = !rowcounter + 1
{%tn}(!rowcounter,1) = "N. Coefs"
{%tn}.setjust(!rowcounter,1) left
for !i=1 to @rows({%ncoefs})
	{%tn}(!rowcounter,1+!i) = {%ncoefs}(!i)
next
!temp = !chosenbreaks+2
{%tn}.settextcolor(!rowcounter,!temp) blue
!rowcounter = !rowcounter + 1

!rowcounter = !rowcounter + 1

{%tn}(!rowcounter,1) = "Chosen number of breaks: " + @str(!chosenbreaks)
!rowcounter = !rowcounter + 1
if !chosenbreaks>0 then
	{%tn}(!rowcounter,1) = "Breaks :"
	{%tn}.setjust(!rowcounter,1) left
	for !i=1 to !chosenbreaks
		{%tn}(!rowcounter,2) = @otods({%breakname}(!i))
		!rowcounter = !rowcounter+1
	next
endif

{%tn}.setlines(!rowcounter) +d
!rowcounter = !rowcounter + 1

_this.display {%tn}

stop


delete {%tn} {%breakname} {%bicname}
if !debug = 0 then
	xclose
endif
smpl {%origsmpl}
